package sun.nio.ch;

import java.nio.channels.SocketChannel;
import java.net.InetSocketAddress;
import java.nio.channels.ServerSocketChannel;
import java.net.InetAddress;
import java.nio.ByteBuffer;
import java.nio.channels.ClosedByInterruptException;
import java.security.PrivilegedExceptionAction;
import java.security.SecureRandom;
import java.security.PrivilegedActionException;
import java.io.IOException;
import java.security.AccessController;
import java.nio.channels.spi.SelectorProvider;
import java.util.Random;
import java.nio.channels.Pipe;

class PipeImpl extends Pipe
{
    private static final int NUM_SECRET_BYTES = 16;
    private static final Random RANDOM_NUMBER_GENERATOR;
    private Pipe.SourceChannel source;
    private Pipe.SinkChannel sink;
    
    PipeImpl(SelectorProvider selectorProvider) throws IOException {
        try {
            AccessController.doPrivileged(new PipeImpl.Initializer(selectorProvider));
        }
        catch (PrivilegedActionException ex) {
            throw (IOException)ex.getCause();
        }
    }
    
    @Override
    public Pipe.SourceChannel source() {
        return this.source;
    }
    
    @Override
    public Pipe.SinkChannel sink() {
        return this.sink;
    }
    
    static {
        RANDOM_NUMBER_GENERATOR = new SecureRandom();
    }
    
    private class Initializer implements PrivilegedExceptionAction<Void>
    {
        private final SelectorProvider sp;
        private IOException ioe;
        
        private Initializer(SelectorProvider sp) {
            this.ioe = null;
            this.sp = sp;
        }
        
        @Override
        public Void run() throws IOException {
            PipeImpl.Initializer.LoopbackConnector loopbackConnector = new PipeImpl.Initializer.LoopbackConnector();
            loopbackConnector.run();
            if ((this.ioe instanceof ClosedByInterruptException)) {
                this.ioe = null;
                Thread thread = new Thread(this, loopbackConnector) {
                    @Override
                    public void interrupt() {
                    }
                };
                thread.start();
                Label_0043: {
                    try {
                        thread.join();
                    }
                    catch (InterruptedException ex) {
                        continue;
                    }
                }
                Thread.currentThread().interrupt();
            }
            if (this.ioe != null) {
                throw new IOException("Unable to establish loopback connection", this.ioe);
            }
            return null;
        }
        
        private class LoopbackConnector implements Runnable
        {
            @Override
            public void run() {
                ServerSocketChannel open = null;
                SocketChannel open2 = null;
                SocketChannel accept = null;
                try {
                    ByteBuffer allocate = ByteBuffer.allocate(16);
                    ByteBuffer allocate2 = ByteBuffer.allocate(16);
                    InetAddress byName = InetAddress.getByName("127.0.0.1");
                    assert !(!byName.isLoopbackAddress());
                    Object remote = null;
                    while (true) {
                        if (open == null || !open.isOpen()) {
                            open = ServerSocketChannel.open();
                            open.socket().bind(new InetSocketAddress(byName, 0));
                            remote = new InetSocketAddress(byName, open.socket().getLocalPort());
                        }
                        open2 = SocketChannel.open(remote);
                        PipeImpl.RANDOM_NUMBER_GENERATOR.nextBytes(allocate.array());
                        do {
                            open2.write(allocate);
                        } while (allocate.hasRemaining());
                        allocate.rewind();
                        accept = open.accept();
                        do {
                            accept.read(allocate2);
                        } while (allocate2.hasRemaining());
                        allocate2.rewind();
                        if (allocate2.equals(allocate)) {
                            break;
                        }
                        accept.close();
                        open2.close();
                    }
                    PipeImpl.this.source = new SourceChannelImpl(PipeImpl.Initializer.this.sp, open2);
                    PipeImpl.this.sink = new SinkChannelImpl(PipeImpl.Initializer.this.sp, accept);
                }
                catch (IOException ex) {
                    try {
                        if (open2 != null) {
                            open2.close();
                        }
                        if (accept != null) {
                            accept.close();
                        }
                    }
                    catch (IOException ex2) {}
                    PipeImpl.Initializer.this.ioe = ex;
                }
                finally {
                    try {
                        if (open != null) {
                            open.close();
                        }
                    }
                    catch (IOException ex3) {}
                }
            }
        }
    }
}
